#!/usr/bin/perl
# [[[ HEADER ]]]
use strict;
use warnings;
use RPerl;
our $VERSION = 0.001_000;

# [[[ CRITICS ]]]
## no critic qw(ProhibitUselessNoCritic ProhibitMagicNumbers RequireCheckedSyscalls)  # USER DEFAULT 1: allow numeric values & print operator
## no critic qw(RequireInterpolationOfMetachars)  # USER DEFAULT 2: allow single-quoted control characters & sigils

# [[[ OPERATIONS ]]]

my integer $i = 10;
FOOBARLOOP:
while ( $i < 23 ) {
    print 'Production rule Statement matched by Loop, iteration number ', $i,
        "\n";
    $i += 2;
    if ( $i > 18 ) {
        last FOOBARLOOP;
    }
}
