use strict;  use warnings;
package RPerl::DataType;

# include modifiers here to be utilized by individual data types
use RPerl::DataType::Modifier::Reference;

# RPerl Data Type, no inheritance from RPerl or RPerl::CompileUnit::Module::Class
1;
